module mod_functions

use global

implicit none

contains

!=================================================================
real(8) function output(meanA)
! function which solves for meanA
real(8),intent(in):: meanA
real(8):: wtmp, numer, denom, Htmp, rtmp, Ktmp
wtmp = (1.0d0-alpha)*((1.0d0/beta-1.0d0+deltak)/alpha)**(alpha/(alpha-1.0d0))
numer= wtmp*meanA**(1.0d0/phi)*phi*deltah**((phi-1.0d0)/phi)
denom=1.0d0/beta-1.0d0+deltah*(1.0d0-phi)
Htmp=(numer/denom)**(phi/(1.0d0-2.0d0*phi))
rtmp=1.0d0/beta-1.0d0
Ktmp=(alpha/(rtmp+deltak))**(1.0d0/(1.0d0-alpha))*Htmp
output=(Ktmp**alpha)*(Htmp**(1.0d0-alpha))-1.0d0

end function output

!=================================================================
real(8) function Eulerxh(xxh,A,h,indA,indk)

real(8),intent(in):: xxh, A,h
integer, intent(in):: indA, indk

integer::i1
real(8):: Edval1next, Edval2next, hptmp, hp, dval1next, dval2next, dval1onh(dimgridh), dval2onh(dimgridh)

Edval1next=0.0d0
Edval2next=0.0d0

do i1=1,ngrid
    hptmp=(1.0d0-deltah)*h+A*h**phi*xxh**phi
    hp=etashock(i1)*hptmp
    
    dval1onh=dval1(indk,:,indA)
    dval1next=interp1nu(dimgridh,hgrid,dval1onh,hp)
    Edval1next=Edval1next+etashockprob(i1)*dval1next
    
    dval2onh=dval2(indk,:,indA)
    dval2next=interp1nu(dimgridh,hgrid,dval2onh,hp)
    Edval2next=Edval2next+etashockprob(i1)*etashock(i1)*dval2next
    
enddo

Eulerxh=xxh**(1.0d0-phi)*Edval1next-A*phi*h**phi*Edval2next

end function Eulerxh
!====================================================================
real(8) function Eulerxhsquare(x,A,h,indA,indk)

real(8),intent(in)::X, A,h
integer, intent(in):: indA, indk

integer::i1
real(8):: Edval1next, Edval2next, hptmp, hp, dval1next, dval2next, dval1onh(dimgridh), dval2onh(dimgridh)

Edval1next=0.0d0
Edval2next=0.0d0

do i1=1,ngrid
    hptmp=(1.0d0-deltah)*h+A*h**phi*X**phi
    hp=etashock(i1)*hptmp
    
    dval1onh=dval1(indk,:,indA)
    dval1next=interp1nu(dimgridh,hgrid,dval1onh,hp)
    Edval1next=Edval1next+etashockprob(i1)*dval1next
    
    dval2onh=dval2(indk,:,indA)
    dval2next=interp1nu(dimgridh,hgrid,dval2onh,hp)
    Edval2next=Edval2next+etashockprob(i1)*etashock(i1)*dval2next
    
enddo

Eulerxhsquare=-(X**(1.0d0-phi)*Edval1next-A*phi*h**phi*Edval2next)**2.0d0

end function Eulerxhsquare

!==================================================================
real(8) function Eulerkink(xxh,A,h,k,indA)

real(8), intent(in):: xxh,A,h,k
integer, intent(in):: indA

integer:: i1
real(8):: Edval2next, hptmp, hp, dval2next, dval2onh(dimgridh), ctmp, cbdd

hptmp=(1.0d0-deltah)*h+A*h**phi*xxh**phi
Edval2next=0.0d0

do i1=1,ngrid
    hp=etashock(i1)*hptmp
    dval2onh=dval2(1,:,indA)
    dval2next=interp1nu(dimgridh,hgrid,dval2onh,hp)
    Edval2next=Edval2next+etashockprob(i1)*etashock(i1)*dval2next
enddo

ctmp=(1.0d0+r)*k+w*h-xxh
cbdd=max(ctmp,0.0001)
Eulerkink=cbdd**(-sigma)-A*phi*h**phi*xxh**(phi-1.0d0)*beta*Edval2next

end function Eulerkink
!==========================================================================

real(8) function Eulerkinksquare(x,A,h,k,indA)

real(8), intent(in):: x,A,h,k
integer, intent(in):: indA

integer:: i1
real(8):: Edval2next, hptmp, hp, dval2next, dval2onh(dimgridh), ctmp, cbdd

hptmp=(1.0d0-deltah)*h+A*h**phi*x**phi
Edval2next=0.0d0

do i1=1,ngrid
    hp=etashock(i1)*hptmp
    dval2onh=dval2(1,:,indA)
    dval2next=interp1nu(dimgridh,hgrid,dval2onh,hp)
    Edval2next=Edval2next+etashockprob(i1)*etashock(i1)*dval2next
enddo

ctmp=(1.0d0+r)*k+w*h-x
cbdd=max(ctmp,0.0001)
Eulerkinksquare=-(cbdd**(-sigma)-A*phi*h**phi*x**(phi-1.0d0)*beta*Edval2next)**2.0d0

end function Eulerkinksquare

!==================================================================

real(8) function Eulerxhnew(xxh,A,h,indA,k,kprime)
! function for solving xh when k'=0
real(8), intent(in):: xxh,A,h,k,kprime
integer, intent(in):: indA

integer:: i1
real(8):: Edval2next, hptmp, hp, dval2next, ctmp, cbdd

hptmp=(1.0d0-deltah)*h+A*h**phi*xxh**phi
Edval2next=0.0d0

do i1=1,ngrid
    hp=etashock(i1)*hptmp
    dval2next=interp2nu(dimgridk,dimgridh,kgrid,hgrid,dval2(:,:,indA),kprime,hp)
    Edval2next=Edval2next+etashockprob(i1)*etashock(i1)*dval2next
enddo

ctmp=(1.0d0+r)*k+w*h-xxh-kprime
cbdd=max(ctmp,0.0001)
Eulerxhnew=cbdd**(-sigma)-A*phi*h**phi*xxh**(phi-1.0d0)*beta*Edval2next

end function Eulerxhnew
!==========================================================================
real(8) function Eulerxhnewsquare(xxh,A,h,indA,k,kprime)
! function for solving s when k'=0
real(8), intent(in):: xxh,A,h,k,kprime
integer, intent(in):: indA

integer:: i1
real(8):: Edval2next, hptmp, hp, dval2next, ctmp, cbdd

hptmp=(1.0d0-deltah)*h+A*h**phi*xxh**phi
Edval2next=0.0d0

do i1=1,ngrid
    hp=etashock(i1)*hptmp
    dval2next=interp2nu(dimgridk,dimgridh,kgrid,hgrid,dval2(:,:,indA),kprime,hp)
    Edval2next=Edval2next+etashockprob(i1)*etashock(i1)*dval2next
enddo

ctmp=(1.0d0+r)*k+w*h-xxh-kprime
cbdd=max(ctmp,0.0001)
Eulerxhnewsquare=-(cbdd**(-sigma)-A*phi*h**phi*xxh**(phi-1.0d0)*beta*Edval2next)**2.0d0

end function Eulerxhnewsquare


!================= linear interpolations============================
! 1-D linear interpolation using uniform grid
real(8) function interp1(dimgrid,xgrid,fx,xi)

integer, intent(in):: dimgrid
real(8), intent(in):: xgrid(dimgrid), fx(dimgrid), xi

real(8):: step, xi_min, ww
integer:: loc

step=(xgrid(dimgrid)-xgrid(1))/(dimgrid-1.0)
xi_min=xi-xgrid(1)
loc=min(dimgrid-1,max(1,floor(xi_min/step)+1))
ww=(xi-xgrid(loc))/(xgrid(loc+1)-xgrid(loc))

interp1=(1-ww)*fx(loc)+ww*fx(loc+1)

end function interp1
!==========================================================
! 1-D linear interpolation using non-uniform grid
real(8) function interp1nu(dimgrid,xgrid,fx,xi)

integer, intent(in):: dimgrid
real(8), intent(in):: xgrid(dimgrid), fx(dimgrid), xi

real(8):: t
integer:: x_min(1), x_ind

x_min=minloc(abs(xgrid-xi))
x_ind=x_min(1)

if (xi<xgrid(x_ind)) x_ind=x_ind-1

x_ind=max(x_ind,1)
x_ind=min(x_ind,dimgrid-1)

t=(fx(x_ind+1)-fx(x_ind))/(xgrid(x_ind+1)-xgrid(x_ind))
interp1nu=t*(xi-xgrid(x_ind))+fx(x_ind)

end function interp1nu

!===========================================================
! 2-D linear linterpolation using unifor grids
real(8) function interp2(xgrid,ygrid,fxy,xi,yi)

real(8),intent(in):: xgrid(dimgridk), ygrid(dimgridh), fxy(dimgridk,dimgridh), xi, yi

real(8):: sx, sy, xi_min, yi_min, wwx, wwy
integer:: locx, locy

sx=(xgrid(dimgridk)-xgrid(1))/(dimgridk-1.0)
xi_min=xi-xgrid(1)
locx=min(dimgridk-1,max(1,floor(xi_min/sx)+1))

sy=(ygrid(dimgridh)-ygrid(1))/(dimgridh-1.0)
yi_min=yi-ygrid(1)
locy=min(dimgridh-1,max(1,floor(yi_min/sy)+1))

wwx=(xi-xgrid(locx))/(xgrid(locx+1)-xgrid(locx))
wwy=(yi-ygrid(locy))/(ygrid(locy+1)-ygrid(locy))

interp2=(1-wwx)*(1-wwy)*fxy(locx,locy)+(1-wwx)*wwy*fxy(locx,locy+1)+wwx*(1-wwy)*fxy(locx+1,locy)+wwx*wwy*fxy(locx+1,locy+1)

end function interp2

!===========================================================
! 2-D linear linterpolation using unifor grids
real(8) function interp2new(dimgridx,dimgridy,xgrid,ygrid,fxy,xi,yi)

integer,intent(in):: dimgridx,dimgridy
real(8),intent(in):: xgrid(dimgridx), ygrid(dimgridy), fxy(dimgridx,dimgridy), xi, yi

real(8):: sx, sy, xi_min, yi_min, wwx, wwy
integer:: locx, locy

sx=(xgrid(dimgridx)-xgrid(1))/(dimgridx-1.0)
xi_min=xi-xgrid(1)
locx=min(dimgridx-1,max(1,floor(xi_min/sx)+1))

sy=(ygrid(dimgridy)-ygrid(1))/(dimgridy-1.0)
yi_min=yi-ygrid(1)
locy=min(dimgridy-1,max(1,floor(yi_min/sy)+1))

wwx=(xi-xgrid(locx))/(xgrid(locx+1)-xgrid(locx))
wwy=(yi-ygrid(locy))/(ygrid(locy+1)-ygrid(locy))

interp2new=(1-wwx)*(1-wwy)*fxy(locx,locy)+(1-wwx)*wwy*fxy(locx,locy+1)+wwx*(1-wwy)*fxy(locx+1,locy)+wwx*wwy*fxy(locx+1,locy+1)

end function interp2new

!===========================================================
! 2-D linear linterpolation using non-uniform grids
real(8) function interp2nu(dimgridx,dimgridy,xgrid,ygrid,fxy,xi,yi)

integer,intent(in):: dimgridx,dimgridy
real(8),intent(in):: xgrid(dimgridx), ygrid(dimgridy), fxy(dimgridx,dimgridy), xi, yi

integer:: x_min(1), x_ind, y_min(1), y_ind
real(8):: wwx, wwy

x_min=minloc(abs(xgrid-xi))
x_ind=x_min(1)

if (xi<xgrid(x_ind)) x_ind=x_ind-1

x_ind=max(x_ind,1)
x_ind=min(x_ind,dimgridx-1)

y_min=minloc(abs(ygrid-yi))
y_ind=y_min(1)

if (yi<ygrid(y_ind)) y_ind=y_ind-1

y_ind=max(y_ind,1)
y_ind=min(y_ind,dimgridy-1)

wwx=(xi-xgrid(x_ind))/(xgrid(x_ind+1)-xgrid(x_ind))
wwy=(yi-ygrid(y_ind))/(ygrid(y_ind+1)-ygrid(y_ind))

interp2nu=(1.0d0-wwx)*(1.0d0-wwy)*fxy(x_ind,y_ind)+(1.0d0-wwx)*wwy*fxy(x_ind,y_ind+1) &
    +wwx*(1.0d0-wwy)*fxy(x_ind+1,y_ind)+wwx*wwy*fxy(x_ind+1,y_ind+1)

end function interp2nu

end module